# -*- coding: utf-8 -*-

__version__ = '0.1.0'

from django.contrib.admin import SimpleListFilter
from django.utils.translation import ugettext_lazy as _


def isnull_filter(field_name, filter_title=None):
    class HasRelatedFieldFilter(SimpleListFilter):
        parameter_name = '%s__isnull' % field_name
        related_field = field_name

        def __init__(self, request, field, model, model_admin):
            related_field = model._meta.get_field(field_name)
            if filter_title:
                self.title = filter_title
            else:
                if hasattr(related_field, 'related'):
                    related_title = related_field.related.model._meta.verbose_name_plural
                    self.title = _("Is related '%s' null?") % related_title
                elif hasattr(related_field, 'related_model') and hasattr(related_field.related_model, "_meta"):
                    related_title = related_field.related_model._meta.verbose_name_plural
                    self.title = _("Is related '%s' null?") % related_title
                else:
                    related_title = related_field.name
                    self.title = _("Is field '%s' null?") % related_title
            super(HasRelatedFieldFilter, self).__init__(request, field, model, model_admin)

        def lookups(self, request, model_admin):
            return [
                ('true', _('True')),
                ('false', _('False')),
            ]

        def queryset(self, request, queryset):
            kwargs = {}
            if self.value() in ('false', 'False'):
                kwargs["%s__isnull" % self.related_field] = False
            if self.value() in ('true', 'True'):
                kwargs["%s__isnull" % self.related_field] = True
            return queryset.filter(**kwargs).distinct()

    return HasRelatedFieldFilter



""" Something here """

from uuid import uuid4

from django.db import models
from django.utils.translation import ugettext_lazy as _

from meeting.common.managers import BaseManager


__all__ = ['NamedBaseModel']


class BaseModel(models.Model):
    objects = BaseManager(this_year_lookup='created_on__year')

    created_on = models.DateTimeField(auto_now_add=True,
                                      verbose_name=_('Criado em'))

    modified_on = models.DateTimeField(auto_now=True,
                                       verbose_name=_('Modificado em '))

    class Meta:
        abstract = True
        ordering = ('created_on', )


class NamedBaseModel(models.Model):
    """Named Something"""

    name = models.CharField(
        max_length=100,
        verbose_name=_('Nome'),
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class PersonModel(models.Model):
    name = models.CharField(
        max_length=100,
        verbose_name=_('Nome'),
    )

    document = models.CharField(
        max_length=100,
        verbose_name=_('Documento'),
    )

    note = models.TextField(
        blank=True,
        verbose_name=_('Nota')
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name


class RawUUIDModel(models.Model):
    uuid = models.CharField(
        blank=True,
        db_index=True,
        null=True,
        max_length=36,
        unique=True,
        editable=False,
        verbose_name=_('QRCode'),
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.uuid

    def save(self, *args, **kwargs):
        if not self.uuid:
            self.uuid = str(uuid4())
        return super().save(*args, **kwargs)



from django.test import TestCase

from example.models import Purchase, PurchaseItem


class ManagersTestCase(TestCase):

    def setUp(self):
        purchase = self.purchase2k14 = Purchase.objects.create()
        purchase.created_on = purchase.created_on.replace(year=2014)
        purchase.save()
        self.purchaseitem2k14 = purchase.purchaseitem_set.create()

        purchase = Purchase.objects.create()
        purchase.created_on = purchase.created_on.replace(year=2015)
        purchase.save()
        purchase.purchaseitem_set.create()

        purchase = self.purchase = Purchase.objects.create()
        self.purchaseitem = purchase.purchaseitem_set.create()

    def test_this_year(self):
        self.assertQuerysetEqual(Purchase.objects.this_year(),
                                 [repr(self.purchase)])

        self.assertQuerysetEqual(PurchaseItem.objects.this_year(),
                                 [repr(self.purchaseitem)])

    def test_from_year(self):
        self.assertQuerysetEqual(Purchase.objects.from_year(2014),
                                 [repr(self.purchase2k14)])

        self.assertQuerysetEqual(PurchaseItem.objects.from_year(2014),
                                 [repr(self.purchaseitem2k14)])


class ModelsTestCase(TestCase):
    def test_uuid_auto_generated(self):
        purchase = Purchase.objects.create()
        self.assertIsNotNone(purchase.uuid)



from django.db.models import Manager, QuerySet
from django.utils import timezone


class BaseManager(Manager):
    def __init__(self, *args, **kwargs):
        self.this_year_lookup = kwargs.pop('this_year_lookup',
                                           'created_on__year')
        super().__init__(*args, **kwargs)

    def get_queryset(self):
        return BaseManagerQuerySet(self.model,
                                   using=self._db,
                                   this_year_lookup=self.this_year_lookup)

    def from_year(self, year, *ar, **kw):
        return self.get_queryset().from_year(year, *ar, **kw)

    def this_year(self, *ar, **kw):
        return self.get_queryset().this_year(*ar, **kw)


class BaseManagerQuerySet(QuerySet):
    def __init__(self, *args, **kwargs):
        self.this_year_lookup = kwargs.pop('this_year_lookup',
                                           'created_on__year')
        super().__init__(*args, **kwargs)

    def from_year(self, year):
        return self.filter(**{self.this_year_lookup: year})

    def this_year(self):
        return self.filter(**{self.this_year_lookup: timezone.now().year})



from setuptools import setup, find_packages


packages = ['meeting.{}'.format(i)
            for i in find_packages('meeting', exclude=['tests'])]

setup(
    name='meeting',
    version=open('VERSION').read().strip(),
    description='Festival management application: '
    'Tickets E-Commerce, '
    'Tickets validation on Lobby (Web, Paper, Guest, '
    'Local Purchase) and '
    'Sells on Wristband/Card (Beverage, Food, Souvenir, etc).',
    long_description=open('README.rst').read().strip(),
    author='Paulo R',
    author_email='proberto.macedo@gmail.com',
    url='http://github.com/mauler/meeting',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
    install_requires=[
        'django',
        'djangorestframework',
    ],
    zip_safe=False,
    keywords='festival rfid qrcode barcode tickets',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries :: '
        'Python Modules'])
